### #!/usr/bin/python

#  - main.py - 
# This file is part of mp3togo

# Convert audio files to play on a mp3 player
#
# (c) Simeon Veldstra 2004, 2006  <reallifesim@gmail.com>
#
# This software is free.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You may redistribute this program under the terms of the
# GNU General Public Licence version 2
# Available in this package or at http://www.fsf.org


"""Convert a list of files.

	Find a list of audio files in various 
	formats and qualities and convert them
	to a uniform format suitable for a 
	portable player."""

import sys
import os
import time
import termios
import fcntl

import mp3togo
import mp3togo.filelist as filelist
import mp3togo.conf as conf
import mp3togo.track as track
import mp3togo.task as task
import mp3togo.pool as pool


def fail(mesg, code=1):
	if mesg:
		print >>sys.stderr, mesg
	sys.exit(code)

def main(argv):

	# Read in configuration
	try:
		opts = conf.Options(argv)
	except conf.Fail, msg:
		print setup.Options().usage()
		fail(str(msg))

	# Check for sanity
	if not opts.bin[opts['encoder']]:
		opts.log(1, "Encoder binary '%s' not found in PATH! Quiting." % opts['encoder'])
		fail('')
	if not opts.bin['normalize-audio'] and not opts['nonormal']:
		opts.log(1, "'normalize-audio' binary not found. Normalization disabled.")
		opts['nonormal'] = True
	if not os.path.isdir(opts['playerdir']):
		opts.log(1, "target_dir does not exist")
		fail('')

	# Build a playlist
	playlist = filelist.FileList()

	if opts['playlist']:
		playlist.addplaylist(opts['playlist'])

	if opts['arg_files']:
		playlist.addfiles(opts['arg_files'])

	if opts['readxmms']:
		playlist.addXMMS()

	# Offer a hint
	if len(playlist) == 0:
		print opts.usage()
		opts.log(1, "No files to convert.")
		fail('')

	# Start work
	execute_sequential(playlist, opts)


def execute_sequential(playlist, opts):
	"""Run the conversions one at a time"""
	print "mp3togo %s\n" % mp3togo.version
	print "<space> or 'p' to pause, <esc> or 'q' to quit\n"
	c = ''
	start_time = time.time()
	good_ones = 0
	bad_ones = 0
	playlist.run()

	try:
		# Terminal sushi
		fd = sys.stdin.fileno()
		oldterm = termios.tcgetattr(fd)
		newattr = termios.tcgetattr(fd)
		newattr[3] = newattr[3] & ~termios.ICANON & ~termios.ECHO
		termios.tcsetattr(fd, termios.TCSANOW, newattr)

		oldflags = fcntl.fcntl(fd, fcntl.F_GETFL)
		fcntl.fcntl(fd, fcntl.F_SETFL, oldflags | os.O_NONBLOCK)

		for name in playlist:
			track_start = time.time()
			print "(%d/%d) %s: " % (playlist.cur_file() + 1, len(playlist), name)
			trk = track.Track(name, opts)
			pl = pool.Pool(opts)
			if pl.add_track(trk):
				tsk = trk.getfirst()
				try:
					while tsk:
						tsk.run()
						if tsk.name in ('Start', 'Done', 'Cleaning'):
							tsk.wait()
							tsk = tsk.next()
							continue
						while tsk.status() == task.RUNNING:
							#parse output
							sys.stdout.write("\r")
							pcent = tsk.output()
							if 0 < pcent < 101:
								bar = '#' * int(pcent/5.0) + ('_' * (20 - int(pcent/5.0)))
							else:
								bar = ""
								pcent = 0
							sys.stdout.write("%s: %3.d%% %s" % (tsk.name, int(pcent), bar))
							# Get keyboard input
							time.sleep(0.01)
							for busy in range(5):
								try:
									c = sys.stdin.read(1)
									break
								except IOError: pass
							if c in ('q', 'Q', chr(27)):
								raise KeyboardInterrupt
							elif c in (' ', 'p', 'P'):
								if tsk.pause():
									sys.stdout.write("\r  [PAUSED] hit <enter> to resume       ")
									while 1:
										try:
											c = sys.stdin.read(1)
											tsk.run()
											break
										except IOError: pass
						tm = tsk.elapsed_time()
						ts = format_time(tm)
						tsk.wait()
						sys.stdout.write("\r                                                 ")
						if tsk.status() == task.DONE:
							if ts:
								print "\r%s: Done. %s elapsed." % (tsk.name, ts)
							else:
								print "\r%s: Done." % tsk.name
							tsk = tsk.next()
						else:
							print "\r%s: Failed. Cleaning up." % tsk.name
							while tsk:
								print "Undo: %s" % tsk.name
								tsk = tsk.prev()
				except KeyboardInterrupt:
					tsk.stop()
					print "Stopping..."
					while tsk:
						print "Undo: %s" % tsk.name
						tsk = tsk.prev()
					sys.exit(1)
					pass
			else:
				opts.log(1, "Out of space. Exiting.")
				break
	
			pl.rm_track(trk)
			if trk() == track.DONE:
				good_ones += 1
			else:
				bad_ones += 1
			trk.close()
			del trk

			tt = time.time() - track_start
			ts = format_time(tt)
			if ts:
				print "Total time this track: %s\n" % ts
			else:
				print 
	
		tm = time.time() - start_time
		ts = format_time(tm)
		if bad_ones:
			if bad_ones == 1:
				bad_str = ", 1 track failed."
			else:
				bad_str = ", %d tracks failed." % bad_ones
		else:
			bad_str = "."
		#bytes = format_bytes(pl.produced_bytes)
		print "\n%d tracks done%s %s total time elapsed." % \
		      (good_ones, bad_str, ts)
		
	finally:
		# Fry some fish:
		termios.tcsetattr(fd, termios.TCSAFLUSH, oldterm)
		fcntl.fcntl(fd, fcntl.F_SETFL, oldflags)


def format_time(tm):
	tm = int(tm)
	if tm:
		hr, min, sec = (tm/3600, (tm%3600)/60, (tm%3600)%60)
		ts = "%ds" % sec
		if min > 0:
			ts = ("%dm" % min) + ts
		if hr > 0:
			ts = ("%dh" % hr) + ts
		return ts
	else:
		return ''

def format_bytes(bytes):
	k = 1024.0
	m = 1024 * k
	g = 1024 * m
	if 0 < bytes < k:
		ret = str(int(bytes/k)) + " KB"
	elif k <= bytes < m:
		ret = str(int(bytes/k)) + " KB"
	elif m <= bytes < g:
		ret = str(int(bytes/m)) + " MB"
	else:
		ret = str(bytes) + " Bytes"

if __name__ == "__main__": main(sys.argv)

