#  - tags.py - 
# This file is part of mp3togo

# Convert audio files to play on a mp3 player
# Manage program options
#
# (c) Simeon Veldstra 2004, 2006  <reallifesim@gmail.com>
#
# This software is free.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You may redistribute this program under the terms of the
# GNU General Public Licence version 2
# Available in this package or at http://www.fsf.org

import mp3togo.conf as setup

if setup.THREADED == True:
	import threading
else:
	import dummy_threading as threading

import os, sys
import UserDict

try:
	import ogg.vorbis
except ImportError:
	HAVE_VORBIS = False
else:
	HAVE_VORBIS = True

try:
	import ID3
except ImportError:
	HAVE_ID3 = False
else:
	HAVE_ID3 = True

HAVE_METAFLAC=False
for path in os.environ['PATH'].split(':'):
	if os.path.exists(os.path.join(path, 'metaflac')):
		HAVE_METAFLAC=True
		break


class Tags(UserDict.DictMixin):
	"""Manage per track metadata.

	Provide a filename in the constructor
	Call read() to read in the tags
	modify tags if necessary
	Call write(name) with the output file the tags
	are to be written to
	Tags are available through a dictionary interface
	format(fmt) returns a formatted string of metadata
	"""

	def __init__(self, filename):
		if not os.path.exists(filename):
			raise setup.ErrorNoFile
		self._file = filename
		self._type = setup.getfiletype(filename)
		self._tags = {}
		self._readok = threading.Event()
		self._readok.clear()
		self._lock = threading.Lock()


	def read(self, block=True):
		if not self._lock.acquire(block) and not block:
			return False
		self._readok.clear()

		def copytags(d):
			o = {}
			for k in d.keys():
				o[k] = list(d[k])
			return o

		if self._type == 'mp3' and HAVE_ID3:
			info = ID3.ID3(self._file, as_tuple=1).as_dict()
			self._tags = copytags(info) 
			del info
		elif self._type == 'ogg' and HAVE_VORBIS:
			info = ogg.vorbis.VorbisFile(self._file).comment().as_dict()
			self._tags = copytags(info) 
			del info
		elif self._type == 'flac' and HAVE_METAFLAC:
			cmd = '%s --export-tags-to=- "%s" ' % ('metaflac', self._file)
			fd = os.popen(cmd)
			info = fd.read()
			fd.close()
			info = map(lambda  x: x.split('=', 1), info.split('\n'))
			info = filter(lambda x: len(x) == 2, info)
			info = map(lambda x: [x[0].upper(), x[1:]], info)
			self._tags = dict(info)
		elif self._type == 'wav':
			pass
			

		# Try to guess from the file's path - better than nothing
		path = os.path.splitext(self._file)[0]
		path = path.replace('_', ' ')
		for id, depth in [('ARTIST', -3), ('ALBUM', -2), ('TITLE', -1)]:
			if not id in self._tags or self._tags[id][0] == '':
				try:
					self._tags[id] = [path.split(os.sep)[depth]]
				except IndexError:
					self._tags[id] = "%s unknown" % id.lower()

		self._readok.set()
		self._lock.release()
		return True


	def write(self, filename, block=True):
		if not self._lock.acquire(block) and not block:
			return False

		if not os.path.exists(filename):
			self._lock.release()
			raise setup.ErrorNoFile

		def puttags(d):
			for key in self._tags.keys():  # Not self.keys()
				if key == 'GENRE' and fmt == 'mp3':
					if type(self._tags[key][0]) is type(1) and 0 <= self._tags[key][0] < 256:
						d[key] = self._tags[key][0]
					elif self._tags[key][0] in map(str, range(256)):
						d[key] = int(self._tags[key][0])
					else:
						d[key] = int(genres.get(self._tags[key][0], '255'))
				else:	
					d[key] = self._tags[key][0]
			# No! don't unlock here dumbass! return from puttags
			return d

		try:
			fmt = setup.getfiletype(filename)
		except setup.ErrorUnknownFileType:
			self._lock.release()
			raise

		if fmt == 'ogg':
			vf = ogg.vorbis.VorbisFile(filename)
			out = vf.comment()
			puttags(out)
			out.write_to(filename)
			del out
			del vf
		elif fmt == 'mp3':
			out = ID3.ID3(filename, as_tuple=1)
			puttags(out)
			out.write()
			del out
		else:
			self._lock.release()
			raise setup.ErrorUnknownFileType

		self._lock.release()
		return True


	def writeindex(self, filename, indexname, block=True):
		if not self._lock.acquire(block) and not block:
			return False

		try:
			ifile = file(indexname, 'a')
			ifile.write(filename + "\n")
			keys = self._tags.keys()
			keys.sort()
			for k in keys:
				for q in self._tags[k]:
					ifile.write("%s: %s\n" % (k, q))
			ifile.write("\n")
			ifile.close()
		except:
			self._lock.release()
			raise

		self._lock.release()
		return True

	def format(self, fmt, block=True):
		"""Pretty print the tag information

		The following format strings apply:
		%%     Literal %
		%n     Track number
		%a     Artist
		%t     Track title
		%l     Album title
		%y     Album release year
		%g     Album genre"""
		if not self._lock.acquire(block) and not block:
			return False

		esc = {'n': 'TRACKNUMBER',
		       'a': 'ARTIST',
			   't': 'TITLE',
			   'l': 'ALBUM',
			   'y': 'YEAR',
			   'g': 'GENRE_NAME'}
			  #'z': Used for literal '%'

		out = ""
		fmt = fmt.replace('%%', '%z')
		fmt = fmt.split('%')
		while fmt:
			out += fmt[0] 
			if len(fmt) <= 1: 
				break
			fmt = fmt[1:]
			code = fmt[0] and fmt[0][0]
			if code == 'z':
				fmt[0] = '%' + fmt[0][1:]
			elif code in esc.keys():
				fmt[0] = self._tags[esc[code]][0] + fmt[0][1:]
			else:
				self._lock.release()
				raise setup.ErrorBadFormat

		self._lock.release()
		return out


	def __getitem__(self, key, block=True):

		if not self._lock.acquire(block) and not block:
			return False

		if key == 'GENRE_NAME':
			if not self._tags.has_key('GENRE'):
				out = ''
			else:
				out = [genrenumbers.get(self._tags['GENRE'][0], self._tags['GENRE'][0])]
		else:
			try:
				out = self._tags[key.upper()]
			except KeyError:
				self._lock.release()
				raise
				
		self._lock.release()
		return out


	def __setitem__(self, key, value, block=True):

		if not self._lock.acquire(block) and not block:
			return False

		if type(value) != type([]):
			value = [value]
		self._tags[key.upper()] = value

		self._lock.release()
		return True


	def setorappend(self, key, value, block=True):
		if not self._lock.acquire(block) and not block:
			return False

		key = key.upper()
		if self._tags.has_key(key):
			self._tags[key].append(value)
		else:
			self[key] = value

		self._lock.release()
		return True


	def __delitem__(self, key, block=True):
		if not self._lock.acquire(block) and not block:
			return False

		try:
			del self._tags[key]
		except KeyError:
			self._lock.release()
			raise	

		self._lock.release()
		return True


	def keys(self, block=True):
		if not self._lock.acquire(block) and not block:
			return False

		if self._tags.has_key('GENRE'):
			out = self._tags.keys() + ['GENRE_NAME']
		else:
			out = self._tags.keys()

		self._lock.release()
		return out


def remove_from_index(trackname, indexname):
	"""Remove an entry from an index file."""
	if not os.path.exists(indexname):
		return True
	fp = file(indexname, 'r')
	lines = fp.readlines()
	fp.close()
	i = 0
	new = []
	while i < len(lines):
		if lines[i] == trackname + '\n':
			while 1:
				i += 1
				if i >= len(lines):
					break
				if lines[i] == '\n':
					i += 1
					break
		if i >= len(lines):
			break
		new.append(lines[i])
		i += 1
	if new:
		fp = file(indexname, 'w')
		fp.writelines(new)
		fp.close()
	else:
		os.unlink(indexname)
	return True


#DATA

genres = {
'Blues': '0',
'Classic Rock': '1',
'Country': '2',
'Dance': '3',
'Disco': '4',
'Funk': '5',
'Grunge': '6',
'Hip-Hop': '7',
'Jazz': '8',
'Metal': '9',
'New Age': '10',
'Oldies': '11',
'Other': '12',
'Pop': '13',
'R&B': '14',
'Rap': '15',
'Reggae': '16',
'Rock': '17',
'Techno': '18',
'Industrial': '19',
'Alternative': '20',
'Ska': '21',
'Death Metal': '22',
'Pranks': '23',
'Soundtrack': '24',
'Euro-Techno': '25',
'Ambient': '26',
'Trip-Hop': '27',
'Vocal': '28',
'Jazz+Funk': '29',
'Fusion': '30',
'Trance': '31',
'Classical': '32',
'Instrumental': '33',
'Acid': '34',
'House': '35',
'Game': '36',
'Sound Clip': '37',
'Gospel': '38',
'Noise': '39',
'Alt. Rock': '40',
'Bass': '41',
'Soul': '42',
'Punk': '43',
'Space': '44',
'Meditative': '45',
'Instrum. Pop': '46',
'Instrum. Rock': '47',
'Ethnic': '48',
'Gothic': '49',
'Darkwave': '50',
'Techno-Indust.': '51',
'Electronic': '52',
'Pop-Folk': '53',
'Eurodance': '54',
'Dream': '55',
'Southern Rock': '56',
'Comedy': '57',
'Cult': '58',
'Gangsta': '59',
'Top 40': '60',
'Christian Rap': '61',
'Pop/Funk': '62',
'Jungle': '63',
'Native American': '64',
'Cabaret': '65',
'New Wave': '66',
'Psychedelic': '67',
'Rave': '68',
'Showtunes': '69',
'Trailer': '70',
'Lo-Fi': '71',
'Tribal': '72',
'Acid Punk': '73',
'Acid Jazz': '74',
'Polka': '75',
'Retro': '76',
'Musical': '77',
'Rock & Roll': '78',
'Hard Rock': '79',
'Folk': '80',
'Folk/Rock': '81',
'National Folk': '82',
'Swing': '83',
'Fusion': '84',
'Bebob': '85',
'Latin': '86',
'Revival': '87',
'Celtic': '88',
'Bluegrass': '89',
'Avantgarde': '90',
'Gothic Rock': '91',
'Progress. Rock': '92',
'Psychedel. Rock': '93',
'Symphonic Rock': '94',
'Slow Rock': '95',
'Big Band': '96',
'Chorus': '97',
'Easy Listening': '98',
'Acoustic': '99',
'Humour': '100',
'Speech': '101',
'Chanson': '102',
'Opera': '103',
'Chamber Music': '104',
'Sonata': '105',
'Symphony': '106',
'Booty Bass': '107',
'Primus': '108',
'Porn Groove': '109',
'Satire': '110',
'Slow Jam': '111',
'Club': '112',
'Tango': '113',
'Samba': '114',
'Folklore': '115',
'Ballad': '116',
'Power Ballad': '117',
'Rhythmic Soul': '118',
'Freestyle': '119',
'Duet': '120',
'Punk Rock': '121',
'Drum Solo': '122',
'A Capella': '123',
'Euro-House': '124',
'Dance Hall': '125',
'Goa': '126',
'Drum & Bass': '127',
'Club-House': '128',
'Hardcore': '129',
'Terror': '130',
'Indie': '131',
'BritPop': '132',
'Negerpunk': '133',
'Polsk Punk': '134',
'Beat': '135',
'Christian Gangsta Rap': '136',
'Heavy Metal': '137',
'Black Metal': '138',
'Crossover': '139',
'Contemporary Christian': '140',
'Christian Rock': '141',
'Merengue': '142',
'Salsa': '143',
'Thrash Metal': '144',
'Anime': '145',
'Jpop': '146',
'Synthpop': '147'}

genrenumbers = {}
for key, value in genres.items():
	genrenumbers[value] = key

#END
