#  - track.py - 
# This file is part of mp3togo

# Convert audio files to play on a mp3 player
# Manage the transform of a single file 
#
# (c) Simeon Veldstra 2006  <reallifesim@gmail.com>
#
# This software is free.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You may redistribute this program under the terms of the
# GNU General Public Licence version 2
# Available in this package or at http://www.fsf.org

import os
import sys

import mp3togo.conf as setup

if setup.THREADED == True:
	import threading
else:
	import dummy_threading as threading

import mp3togo.tags as tags
import mp3togo.task as task
import mp3togo.helpers 

helpers = mp3togo.helpers.helpers


READY = "ready"
RUNNING = "running"
DONE = 0
FAILED = None

DOGGFACTOR = 10
DMP3FACTOR = 10
DFLACFACTOR = 3

def cache_out(out, cache=['']):
	cache[0] += out
	return cache[0]


class Track:
	"""Encapsulate the transformation of a file."""

	def __init__(self, filename, opts):

		self.bytes = 0
		self._filename = filename
		self._queue = ()
		self._state = FAILED

		def undo_decode():
			if os.path.exists(self._wavname):
				os.unlink(self._wavname)
			return True

		def undo_encode():
			if os.path.exists(self._outname):
				os.unlink(self._outname)
			return True

		# Sentinel tasks 
		def finish(): 
			if os.path.exists(self._outname):
				self.bytes = os.stat(self._outname).st_size
			self._state = DONE
			return DONE
		def start(): 
			self.bytes = 0
			self._state = RUNNING
			return RUNNING
		def abort(): 
			self._state = FAILED
			return FAILED
		job = task.SimpleTask(self, start, None, abort, name="Start")
		tasks = [job]
		del job

		# Names 
		filetype = setup.getfiletype(filename)
		dir, base = os.path.split(filename)
		base = os.path.splitext(base)[0]
		base = base + opts['brwarning']
		base = opts.cleanfilename(base)
		if opts['treedepth']:
			head, dir = os.path.split(dir)
			for i in range(opts['treedepth'] -1):
				head, tail = os.path.split(head)
				dir = os.path.join(tail, dir)
				dir = opts.cleanfilename(dir)
		else:
			dir = ''
		self._outdir = os.path.join(opts['playerdir'], dir)
		self._outname = os.path.join(self._outdir, base) 
		self._outname += '.' + helpers[opts['encoder']]['type']
		self._wavname = os.path.join(opts['tempdir'], base) + '.wav'

		def make_dirs():
			if not os.path.isdir(self._outdir):
				os.makedirs(self._outdir)
			return True

		def rm_dirs():
			if opts['treedepth']:
				tail = self._outdir
				for i in range(opts['treedepth']):
					try:
						os.rmdir(tail)
					except OSError:
						return True
					tail, head = os.path.split(tail)
			return True
						
		job = task.SimpleTask(self, make_dirs, None, 
		                      rm_dirs, name="Creating dirs")
		tasks.append(job)
		del job

		# Decode
		prog = mp3togo.helpers.find_helper(filetype, 'decode')
		tmpreq = mp3togo.helpers.est_decoded_size(self._filename)
		jobname = "Decoding %s" % filetype
		if callable(helpers[prog]['cmd']):
			# SimpleTask
			func = helpers[prog]['cmd'](self._filename, self._wavname)
			job = task.SimpleTask(self, func, None, undo_decode,
			                tmpsize=tmpreq, name=jobname)
		else:
			# Task
			args = mp3togo.helpers.make_args(prog, self._filename, 
			                                 self._wavname)
			job = task.Task(self, args, helpers[prog]['parser'], 
			                undo_decode, tmpsize=tmpreq, name=jobname)
		tasks.append(job)
		del job
		
		# Normalize
		if opts.bin['normalize-audio']:
			ncmd = [opts.bin['normalize-audio'], self._wavname]
			job = task.Task(self, ncmd, lambda: '',
							lambda: True, name="Normalizing")
			tasks.append(job)
			del job
		else:
			if not opts['nonormal']:
				opts.log(2, "'normalize-audio' binary not found, skipping.")

		# Encode
		encoder = opts['encoder']
		prog = helpers[encoder]
		outreq = tmpreq / opts['compfactor']
		jobname = "Encoding %s" % prog['type']
		filter = prog['parser']
		if callable(prog['cmd']):
			func = prog['cmd'](self._wavname, self._outname)
			job = task.SimpleTask(self, func, None, undo_encode,
			                outsize=outreq, name=jobname)
		else:
			# Task
			args = mp3togo.helpers.make_args(encoder, self._wavname,
			                                 self._outname, opts['encopts'])
			job = task.Task(self, args, filter, 
			                undo_encode, outsize=outreq, name=jobname)
		tasks.append(job)
		del job

		# Clean up wav
		job = task.SimpleTask(self, undo_decode, None, undo_decode, 
		                      tmpsize=-tmpreq, 
							  name='Cleaning')
		tasks.append(job)
		del job
		
		# Read the tags
		self.tags = tags.Tags(filename)
		self.tags.read()

		# Write index file
		if opts['index']:
			indexname = os.path.join(self._outdir, '2go.index')
			def undo_index():
				tags.remove_from_index(self._outname, indexname)
				return True
			def write_index():
				self.tags.writeindex(self._outname, indexname)
				return True
			job = task.SimpleTask(self, write_index, None,
								  undo_index, name='Writing index')
			tasks.append(job)
			del job

		# Tag
		def tag_output():
			self.tags.write(self._outname)
			return True
		job = task.SimpleTask(self, tag_output, None, 
		                      lambda: True, name="Tagging")
		tasks.append(job)
		del job

		# Completion sentinel
		job = task.SimpleTask(self, finish, None, start, name="Done")
		tasks.append(job)
		del job

		# Consider the track done if the output file exists:
		if os.path.exists(self._outname) and not opts['force']:
			opts.log(1, "Skipping existing file: %s\n" % self._outname)
			self._queue = tuple(tasks)
			for tsk in self._queue:
				tsk._status = task.DONE
			self._state = DONE
			return 

		# Ready to go
		self._queue = tuple(tasks)
		self._state = READY
		return 

	def tasks(self):
		return  self._queue

	def getfirst(self):
		return  self._queue[0]

	def __call__(self):
		return self._state

	def __del__(self):
		self.close()

	def close(self):
		for child in self._queue:
			child._parent = None
		self._queue = ()

	
